-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxiliary"

local onCouplerAttachedMain = onCouplerAttached
local onCouplerDetachedMain = onCouplerDetached

local latchData = {
    frontLeft = false,
    frontRight = false,
    rearLeft = false,
    rearRight = false,
}

local motor = {}

local drivingStats = {}
-- Economy
-- Normal
-- Aggressive

local warnTimer = -1
local function updateGFX(dt)
end

M.selectedMenu = 0

local maxPages = 5
local justToggledMenu = false
local driveHabitTimer = 0

local startUpTimer = -1
local prevIgnitionLevel = 0


local function updateGaugeData(moduleData, dt)

    local wheelspeed = electrics.values.wheelspeed
    local isMoving = wheelspeed > 0.5
    local powerDisplay = 0


    powerDisplay = powerDisplay + (motor.throttle or 0)
    powerDisplay = powerDisplay - (motor.regenThrottle or 0)

    powerDisplay = (powerDisplay / 1 * (isMoving and 1 or 0))
    moduleData.powerDisplay = powerDisplay


    if(electrics.values.ignitionLevel == 2 and prevIgnitionLevel ~= 2 and isCameraCockpit()) then
        startUpTimer = 0
        --sounds.playSoundOnceFollowNode("SunburstEVStartupSound", 274, 1) -- 274 = steering wheel node
    end
    if(startUpTimer >= 0) then
        startUpTimer = startUpTimer + 1
    end
    if(math.abs(startUpTimer - 20) < 5 or startUpTimer == -1) then
        moduleData.readyLight = true
    else
        moduleData.readyLight = false
    end

    if(startUpTimer > 30) then startUpTimer = -1 end


    driveHabitTimer = driveHabitTimer + dt
    if(driveHabitTimer > 5 and isMoving and not electrics.values.cruiseControlActive) then
        if(powerDisplay >= 0.5) then
            drivingStats['aggressive'] = drivingStats['aggressive'] + 1
        elseif(powerDisplay >= 0.25) then
            drivingStats['normal'] = drivingStats['normal'] + 1
        else --whether no throttle or regen, you are driving economy
            drivingStats['economy'] = drivingStats['economy'] + 1            
        end
        driveHabitTimer = 0
        --dump(drivingStats)
    end    

    moduleData.drivingStats = drivingStats
    moduleData.inParkWarning = false

    moduleData.latchData = latchData
    
    if(electrics.values.toggleMenu == 1 and not justToggledMenu and electrics.values.ignitionLevel ~= 0) then
        if(isCameraCockpit()) then
            sounds.playSoundOnceFollowNode("SunburstEVClickSound", 274, 0.2) -- 274 = steering wheel node
        end
        M.selectedMenu = M.selectedMenu + 1
        if(M.selectedMenu > maxPages-1) then M.selectedMenu = 0 end
        justToggledMenu = true
    end
    if(electrics.values.toggleMenu == 0) then justToggledMenu = false end

    if(electrics.values.ignitionLevel == 0) then
        M.selectedMenu = 0
    end

    moduleData.menuData = {
        curMenu = M.selectedMenu,
        maxPages = maxPages,
    }

    moduleData.cruiseData = {}

    if(warnTimer ~= -1) then
        warnTimer = warnTimer + dt
        if(warnTimer > 2) then warnTimer = -1 end
    end

    if(latchData['frontLeft'] or latchData['frontRight'] or latchData['rearLeft'] or latchData['rearRight']) then
        moduleData.doorOpenWarning = true
        if(warnTimer == -1 and isCameraCockpit()) then
            sounds.playSoundOnceFollowNode("SunburstEVWarningSound", 274, 1) -- 274 = steering wheel node
            warnTimer = 0
        end
    end

    if(electrics.values.gear == "P" and electrics.values.throttle > 0) then
        moduleData.inParkWarning = true
        if(warnTimer == -1 and isCameraCockpit()) then
            sounds.playSoundOnceFollowNode("SunburstEVWarningSound", 274, 1) -- 274 = steering wheel node
            warnTimer = 0
        end
    end

    moduleData.driveMode = controller.getController('driveModes').getCurrentDriveModeKey()

    local count = tableSizeC(v.data.wheels)

    local pressures = {}

    for i = 0, count - 1, 1 do
        local wd = v.data.wheels[i]
        if v.data.pressureGroups[wd.pressureGroup] ~= nil then
            local kpa = obj:getGroupPressure(v.data.pressureGroups[wd.pressureGroup]) * 0.001 
            pressures[wd.name] = kpa
        end
    end

    moduleData.tirePressureData = pressures
    prevIgnitionLevel = electrics.values.ignitionLevel
end

local function setupGaugeData(properties)
    --updatePowerDisplay = properties.powerDisplay or false
    motor = powertrain.getDevice("mainEngine")
end

local function reset()
    M.selectedMenu = 0
    startUpTimer = -1
    prevIgnitionLevel = 0
    drivingStats['economy'] = 0
    drivingStats['normal'] = 0
    drivingStats['aggressive'] = 0
end

local function init(jbeamData)
    M.selectedMenu = 0
    startUpTimer = -1
    prevIgnitionLevel = 0
    drivingStats['economy'] = 0
    drivingStats['normal'] = 0
    drivingStats['aggressive'] = 0
end


local function initSounds(jbeamData)
    sounds.createSFXSource("art/sound/warning.ogg", 'AudioDefault3D', "SunburstEVWarningSound", sounds.engineNode)
    sounds.createSFXSource("art/sound/click_sbev.ogg", 'AudioDefault3D', "SunburstEVClickSound", sounds.engineNode)
    sounds.createSFXSource("art/sound/startup_sbev.ogg", 'AudioDefault3D', "SunburstEVStartupSound", sounds.engineNode)
    print("Init Sounds")
end

function onCouplerAttached(nodeId, obj2id, obj2nodeId, attachSpeed, attachEnergy)
    onCouplerAttachedMain(nodeId, obj2id, obj2nodeId, attachSpeed, attachEnergy) --call normal function from BeamNG
    local n = v.data.nodes[nodeId]
    if(n.partOrigin == "sunburst_door_FR") then
        latchData['frontRight'] = false
    end
    if(n.partOrigin == "sunburst_door_FL") then
        latchData['frontLeft'] = false
    end
    if(n.partOrigin == "sunburst_door_RR") then
        latchData['rearRight'] = false
    end
    if(n.partOrigin == "sunburst_door_RL") then
        latchData['rearLeft'] = false
    end
end

function onCouplerDetached(nodeId, obj2id, obj2nodeId, breakForce)
    onCouplerDetachedMain(nodeId, obj2id, obj2nodeId, breakForce) --call normal function from BeamNG
    local n = v.data.nodes[nodeId]
    if(n.partOrigin == "sunburst_door_FR") then
        latchData['frontRight'] = true
    end
    if(n.partOrigin == "sunburst_door_FL") then
        latchData['frontLeft'] = true
    end
    if(n.partOrigin == "sunburst_door_RR") then
        latchData['rearRight'] = true
    end
    if(n.partOrigin == "sunburst_door_RL") then
        latchData['rearLeft'] = true
    end
end

function isCameraCockpit()
    if obj:getCameraPosition():distance(obj:getPosition() + obj:getNodePosition(beamstate.nodeNameMap.driver)) < 0.5 then
        return true
    end
    return false
end

M.init = init
M.reset = reset
M.updateGFX = updateGFX
M.initSounds = initSounds

M.setupGaugeData = setupGaugeData
M.updateGaugeData = updateGaugeData

return M
